window.Gleam.initSearch([{"doc":"clad","title":"clad","content":"# clad\n\n\n[![Package Version](https://img.shields.io/hexpm/v/clad)](https://hex.pm/packages/clad)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/clad/)\n\n\nCommand line argument decoders for Gleam.\n\n- Clad provides primitives to build a `dynamic.Decoder` for command line arguments.\n- Arguments can be specified with long names (`--name`) or short names (`-n`).\n- Values are decoded in the form `--name value` or `--name=value`.\n- Boolean flags do not an explicit value. If the flag exists it is `True`, and if it is missing it is `False`. (i.e. `--verbose`)\n\n\n## Usage\n\n```sh\ngleam add clad\n```\n\nThis program is in the [examples directory](https://github.com/ryanmiville/clad/tree/main/test/examples)\n\n```gleam\nimport argv\nimport clad\nimport gleam/io\nimport gleam/list\nimport gleam/string\n\ntype Args {\n  Args(name: String, count: Int, scream: Bool)\n}\n\nfn greet(args: Args) {\n  let greeting = case args.scream {\n    True -&gt; &quot;HEY &quot; &lt;&gt; string.uppercase(args.name) &lt;&gt; &quot;!&quot;\n    False -&gt; &quot;Hello, &quot; &lt;&gt; args.name &lt;&gt; &quot;.&quot;\n  }\n  list.repeat(greeting, args.count) |&gt; list.each(io.println)\n}\n\nfn args_decoder() {\n  use name &lt;- clad.string(long_name: &quot;name&quot;, short_name: &quot;n&quot;)\n  use count &lt;- clad.int_with_default(\n    long_name: &quot;count&quot;,\n    short_name: &quot;c&quot;,\n    default: 1,\n  )\n  use scream &lt;- clad.bool(long_name: &quot;scream&quot;, short_name: &quot;s&quot;)\n  clad.decoded(Args(name:, count:, scream:))\n}\n\npub fn main() {\n  let args =\n    args_decoder()\n    |&gt; clad.decode(argv.load().arguments)\n\n  case args {\n    Ok(args) -&gt; greet(args)\n    _ -&gt;\n      io.println(\n        &quot;\nOptions:\n  -n, --name &lt;NAME&gt;    Name of the person to greet\n  -c, --count &lt;COUNT&gt;  Number of times to greet [default: 1]\n  -s, --scream         Whether or not to scream greeting\n      &quot;,\n      )\n  }\n}\n```\n\nRun the program\n\n```sh\n❯ gleam run -m examples/greet -- -n Joe\nHello, Joe.\n❯ gleam run -m examples/greet -- --name=Joe\nHello, Joe.\n❯ gleam run -m examples/greet -- --name Joe --count 3 --scream\nHEY JOE!\nHEY JOE!\nHEY JOE!\n❯ gleam run -m examples/greet\n\nOptions:\n  -n, --name &lt;NAME&gt;    Name of the person to greet\n  -c, --count &lt;COUNT&gt;  Number of times to greet [default: 1]\n  -s, --scream         Whether or not to scream greeting\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/clad&gt;.\n","url":"index.html"},{"doc":"clad","title":"bool","content":"pub fn bool(\n  long_name long_name: String,\n  short_name short_name: String,\n  then next: fn(Bool) -&gt;\n    fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n A decoder that decodes Bool arguments.\n Missing Bool arguments default to `False`.\n # Examples\n \n // data: [&quot;-v&quot;]\n use verbose &lt;- clad.bool(long_name: &quot;verbose&quot;, short_name: &quot;v&quot;)\n // -&gt; True\n \n \n // data: []\n use verbose &lt;- clad.bool(long_name: &quot;verbose&quot;, short_name: &quot;v&quot;)\n // -&gt; False\n \nSynonyms:\nclad.bool\nclad bool","url":"clad.html#bool"},{"doc":"clad","title":"bool_with_default","content":"pub fn bool_with_default(\n  long_name long_name: String,\n  short_name short_name: String,\n  default default: Bool,\n  then next: fn(Bool) -&gt;\n    fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n A decoder that decodes Bool arguments. Assigns a default value if the\n argument is missing.\n \n This function is only necessary if you want to assign the default value as `True`.\n # Examples\n \n // data: []\n use verbose &lt;- clad.bool(\n   long_name: &quot;verbose&quot;, \n   short_name: &quot;v&quot;,\n   default: True,\n )\n // -&gt; True\n \nSynonyms:\nclad.bool_with_default\nclad bool_with_default","url":"clad.html#bool_with_default"},{"doc":"clad","title":"decode","content":"pub fn decode(\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n  arguments: List(String),\n) -&gt; Result(a, List(DecodeError))\n Run a decoder on a list of command line arguments, decoding the value if it\n is of the desired type, or returning errors.\n\n This function pairs well with the [argv package](https://github.com/lpil/argv).\n\n # Examples\n \n {\n   use name &lt;- clad.string(&quot;name&quot;, &quot;n&quot;)\n   use email &lt;- clad.string(&quot;email&quot;, &quot;e&quot;),\n   clad.decoded(SignUp(name:, email:))\n }\n |&gt; clad.decode([&quot;-n&quot;, &quot;Lucy&quot;, &quot;--email=lucy@example.com&quot;])\n // -&gt; Ok(SignUp(name: &quot;Lucy&quot;, email: &quot;lucy@example.com&quot;))\n \n with argv:\n \n {\n   use name &lt;- clad.string(&quot;name&quot;, &quot;n&quot;)\n   use email &lt;- clad.string(&quot;email&quot;, &quot;e&quot;),\n   clad.decoded(SignUp(name:, email:))\n }\n |&gt; clad.decode(argv.load().arguments)\n \nSynonyms:\nclad.decode\nclad decode","url":"clad.html#decode"},{"doc":"clad","title":"decoded","content":"pub fn decoded(\n  value: a,\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n Creates a decoder which directly returns the provided value. \n Used to collect decoded values into a record.\n # Examples\n \n pub fn user_decoder() {\n   use name &lt;- clad.string(&quot;name&quot;, &quot;n&quot;)\n   clad.decoded(User(name:))\n }\n \nSynonyms:\nclad.decoded\nclad decoded","url":"clad.html#decoded"},{"doc":"clad","title":"float","content":"pub fn float(\n  long_name long_name: String,\n  short_name short_name: String,\n  then next: fn(Float) -&gt;\n    fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n A decoder that decodes Float arguments.\n # Examples\n \n // data: [&quot;--price&quot;, &quot;2.50&quot;]\n use price &lt;- clad.float(long_name: &quot;price&quot;, short_name: &quot;p&quot;)\n // -&gt; 2.5\n \nSynonyms:\nclad.float\nclad float","url":"clad.html#float"},{"doc":"clad","title":"float_with_default","content":"pub fn float_with_default(\n  long_name long_name: String,\n  short_name short_name: String,\n  default default: Float,\n  then next: fn(Float) -&gt;\n    fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n A decoder that decodes Float arguments. Assigns a default value if the\n argument is missing.\n # Examples\n \n // data: []\n use price &lt;- clad.float(\n   long_name: &quot;price&quot;,\n   short_name: &quot;p&quot;,\n   default: 2.50,\n )\n // -&gt; 2.5\n \nSynonyms:\nclad.float_with_default\nclad float_with_default","url":"clad.html#float_with_default"},{"doc":"clad","title":"int","content":"pub fn int(\n  long_name long_name: String,\n  short_name short_name: String,\n  then next: fn(Int) -&gt;\n    fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n A decoder that decodes Int arguments.\n # Examples\n \n // data: [&quot;-c&quot;, &quot;2&quot;]\n use count &lt;- clad.int(long_name: &quot;count&quot;, short_name: &quot;c&quot;)\n // -&gt; 2\n \nSynonyms:\nclad.int\nclad int","url":"clad.html#int"},{"doc":"clad","title":"int_with_default","content":"pub fn int_with_default(\n  long_name long_name: String,\n  short_name short_name: String,\n  default default: Int,\n  then next: fn(Int) -&gt;\n    fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n A decoder that decodes Int arguments. Assigns a default value if the\n argument is missing.\n # Examples\n \n // data: []\n use count &lt;- clad.int(\n   long_name: &quot;count&quot;,\n   short_name: &quot;c&quot;,\n   default: 2,\n )\n // -&gt; 2\n \nSynonyms:\nclad.int_with_default\nclad int_with_default","url":"clad.html#int_with_default"},{"doc":"clad","title":"string","content":"pub fn string(\n  long_name long_name: String,\n  short_name short_name: String,\n  then next: fn(String) -&gt;\n    fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n A decoder that decodes String arguments.\n # Examples\n \n // data: [&quot;--name&quot;, &quot;Lucy&quot;]\n use name &lt;- clad.string(long_name: &quot;name&quot;, short_name: &quot;n&quot;)\n // -&gt; &quot;Lucy&quot;\n \nSynonyms:\nclad.string\nclad string","url":"clad.html#string"},{"doc":"clad","title":"string_with_default","content":"pub fn string_with_default(\n  long_name long_name: String,\n  short_name short_name: String,\n  default default: String,\n  then next: fn(String) -&gt;\n    fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n A decoder that decodes String arguments. Assigns a default value if the\n argument is missing.\n # Examples\n \n // data: []\n use name &lt;- clad.string(\n   long_name: &quot;name&quot;,\n   short_name: &quot;n&quot;,\n   default: &quot;Lucy&quot;,\n )\n // -&gt; &quot;Lucy&quot;\n \nSynonyms:\nclad.string_with_default\nclad string_with_default","url":"clad.html#string_with_default"},{"doc":"clad","title":"clad","content":" This module encodes a list of command line arguments as a `dynamic.Dynamic` and\n provides primitives to build a `dynamic.Decoder` to decode records from command line\n arguments.\n\n Arguments are parsed from long names (`--name`) or short names (`-n`). \n Values are decoded in the form `--name value` or `--name=value`. \n Boolean flags do not need an explicit value. If the flag exists it is `True`,\n and `False` if it is missing. (i.e. `--verbose`)\n\n # Examples\n\n ## Encoding\n\n All of the following get encoded the same:\n\n ```sh\n --name Lucy --count 3 --verbose\n --name Lucy --count 3 --verbose true\n --name=Lucy --count=3 --verbosetrue\n ```\n\n ```gleam\n // {&quot;--name&quot;: &quot;Lucy&quot;, &quot;--count&quot;: 3, &quot;--verbose&quot;: true}\n ```\n\n Clad encodes the arguments without any knowledge of your target record. Therefore \n missing Bool arguments are not encoded at all:\n\n ```sh\n --name Lucy --count 3\n ```\n ```gleam\n // {&quot;--name&quot;: &quot;Lucy&quot;, &quot;--count&quot;: 3}\n ```\n\n There is no way to know that a long name and a short name are the same argument when encoding.\n So they are encoded as separate fields:\n\n ```sh\n --name Lucy -n Joe\n ```\n ```gleam\n // {&quot;--name&quot;: &quot;Lucy&quot;, &quot;-n&quot;: &quot;Joe&quot;}\n ```\n\n ## Decoding Fields\n\n Clad provides decoders for `String`, `Int`, `Float`, and `Bool` fields.\n\n Clad&#39;s `bool` decoder assumes missing Bool arguments are `False`:\n\n ```sh\n --name Lucy --count 3\n ```\n ```gleam\n use verbose &lt;- clad.bool(long_name: &quot;verbose&quot;, short_name: &quot;v&quot;)\n // -&gt; False\n ```\n\n Clad&#39;s decoders decode the long name first, then the short name\n if the long name is missing:\n ```sh\n --name Lucy -n Joe\n ```\n ```gleam\n use name &lt;- clad.string(long_name: &quot;name&quot;, short_name: &quot;n&quot;)\n // -&gt; &quot;Lucy&quot;\n ```\n\n It&#39;s common for CLI&#39;s to have default values for arguments. Clad provides `_with_default` functions for this:\n\n ```sh\n --name Lucy\n ```\n ```gleam\n use count &lt;- clad.int_with_default(\n   long_name: &quot;count&quot;,\n   short_name: &quot;c&quot;,\n   default: 1,\n )\n // -&gt; 1\n ```\n ## Decoding Records\n Clad&#39;s API is heavily inspired by (read: copied from) [toy](https://github.com/Hackder/toy).\n ```gleam\n fn arg_decoder() {\n   use name &lt;- clad.string(&quot;name&quot;, &quot;n&quot;)\n   use count &lt;- clad.int_with_default(&quot;count&quot;, &quot;c&quot;, 1)\n   use verbose &lt;- clad.bool(&quot;verbose&quot;, &quot;v&quot;)\n   clad.decoded(Args(name:, count:, verbose:))\n }\n ```\n\n And use it to decode the arguments:\n ```gleam\n // arguments: [&quot;--name&quot;, &quot;Lucy&quot;, &quot;--count&quot;, &quot;3&quot;, &quot;--verbose&quot;]\n\n let args = \n   arg_decoder() \n   |&gt; clad.decode(arguments)\n let assert Ok(Args(&quot;Lucy&quot;, 3, True)) = args\n ```\n\n Here are a few examples of arguments that would decode the same:\n\n ```sh\n --name Lucy --count 3 --verbose\n --name=Lucy -c 3 -v=true\n -n=Lucy -c=3 -v\n ``` ","url":"clad.html"}]);